package org.ow2.petals.se.pojo.listener;

import static org.ow2.petals.component.framework.util.ClassLoaderUtil.createClassLoader;
import java.util.Map;
import javax.jbi.servicedesc.ServiceEndpoint;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.su.ServiceUnitManager;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.se.pojo.Pojo;
import org.ow2.petals.se.pojo.PojoSE;

public class JBIListener extends AbstractJBIListener {

	@Override
	protected void onJBIMessage(Exchange exchange) {
		Map<ServiceEndpoint, Pojo> pojos = ((PojoSE) getComponent()).getPojos();
		Pojo pojo = pojos.get(exchange.getEndpoint());
		if (pojo == null) {
			try {
				getLogger().info("POJO nod loaded, create it.");
				pojo = createPojoForCurrentEndpoint(exchange.getEndpoint());
				pojos.put(exchange.getEndpoint(), pojo);
			} catch (Exception e) {
				getLogger().severe(e.getMessage());
				exchange.setError(e);
			}
		}
		if(pojo != null){
			try {
				pojo.callOnExchangeMethod(exchange);
			} catch (Exception e) {
				getLogger().severe(e.getMessage());
				exchange.setError(e);
			}
		}
	}

	private Pojo createPojoForCurrentEndpoint(ServiceEndpoint endpoint)
			throws Exception {
		// retrieve pojo' class name
		String className = null;
		className = getExtensions().getValue("class-name");
		if (className == null) {
			throw new Exception(
					"'class-name' property not found in the JBI.XML descriptor");
		}

		// create a classloader from the SU path
		final String suRootPath = ((ServiceUnitManager) this.getComponent()
				.getServiceUnitManager())
				.getSUInstallRootForActivatedEp(endpoint);

		ClassLoader classLoader = createClassLoader(suRootPath, this.getClass()
				.getClassLoader());

		// load the pojo class from the classloader
		Class<?> clazz = null;
		try {
			clazz = classLoader.loadClass(className);
		} catch (ClassNotFoundException e) {
			throw new Exception(className
					+ " class not found in the service-unit libraries.", e);
		}

		// create the pojo
		Pojo pojo = null;
		try {
			pojo = new Pojo(clazz.newInstance(), classLoader, getComponent()
					.getContext(), getComponent().getChannel(), getComponent()
					.getLogger());
		} catch (Exception e) {
			throw new Exception(className
					+ " found, but can not create an instance.", e);
		}

		// init the pojo
		pojo.init();

		return pojo;
	}
}
